@@ -1,4 +1,3 @@
-
 """Abstract classes and utility operations for building graph representations and
 preprocessing loaders."""
 
@@ -7,14 +6,16 @@ from typing import Union, Dict, List, Any
 
 import numpy as np
 from mgetool.tool import parallelize, batch_parallelize
-
 from pymatgen.analysis.local_env import NearNeighbors
 from pymatgen.core import Structure
 
+from featurebox.featurizers.atom.mapper import AtomPymatgenPropMap, AtomJsonMap, get_atom_fea_number, get_atom_fea_name
 from featurebox.featurizers.base_transform import DummyConverter, BaseFeature, Converter
-from featurebox.featurizers.bond import BaseBondGet, BondGaussianConverter
-from featurebox.featurizers.local_env import get_nn_strategy
-from featurebox.featurizers.mapper import AtomPymatgenPropMap, AtomJsonMap
+from featurebox.featurizers.bond.expander import BondGaussianConverter
+from featurebox.featurizers.envir.desc_env import DesDict
+from featurebox.featurizers.envir.environment import BaseNNGet, _BaseEnvGet, BaseDesGet
+from featurebox.featurizers.envir.local_env import NNDict
+from featurebox.utils.look_json import get_marked_class
 
 
 def itemgetter_list(data_list: List, indices: List) -> tuple:
@@ -45,18 +46,50 @@ class _StructureGraph(BaseFeature):
     """
 
     def __init__(self, nn_strategy: Union[str, NearNeighbors] = "MinimumDistanceNNAll",
+                 bond_generator: [_BaseEnvGet, str] = None,
                  atom_converter: Converter = None,
                  bond_converter: Converter = None,
+                 state_converter: Converter = None,
+                 return_bonds: str = "all",
                  **kwargs):
         """
-
+        Args:
+            nn_strategy (str): NearNeighbor strategy
+                For bond_converter ="BaseNNGe": ["BrunnerNN_reciprocal","BrunnerNN_real","BrunnerNN_relative",
+                "EconNN", "CrystalNN",]
+                For bond_converter ="BaseDesGet": ["ACSF","BehlerParrinello","EAD","EAMD","SOAP",
+                   "SO3","SO4_Bispectrum","wACSF"]
+            atom_converter (Converter): atom features converter.
+            bond_converter (Converter): bond features converter.
+            state_converter (Converter): state features converter.
+            bond_generator (_BaseEnvGet, str): bond features converter.
+                1, BaseDesGet or 2, BaseDesGet. or there name.
+                if object offered, rather str, the nn_strategy would use the nn_strategy in Converter.
+            cutoff (float): cutoff radius
+                if offered for "BaseNNGet", the nn_strategy would be neglect and find neighbors using
+                ``find_points_in_spheres`` in pymatgen.
+            return_bonds:"all","bonds","bond_state"
+                which bond property return. default "all".
+            **kwargs:
         """
-        super().__init__(**kwargs)
 
-        self.nn_strategy = get_nn_strategy(nn_strategy)
-        self.bond_generator = BaseBondGet(self.nn_strategy)
+        super().__init__(**kwargs)
+        self.return_bonds = return_bonds
+
+        fuk = {"BaseNNGet": BaseNNGet, "BaseDesGet": BaseDesGet}
+        fuv = {"BaseNNGet": NNDict, "BaseDesGet": DesDict, }
+        if bond_generator is None:
+            self.nn_strategy = get_marked_class(nn_strategy, NNDict)
+            self.bond_generator = BaseNNGet(self.nn_strategy)
+        elif isinstance(bond_generator, str):
+            self.nn_strategy = get_marked_class(nn_strategy, fuv[bond_generator])
+            self.bond_generator = fuk[bond_generator](self.nn_strategy)
+        else:
+            self.bond_generator = bond_generator
+            self.nn_strategy = self.bond_generator.nn_strategy
         self.atom_converter = atom_converter or self._get_dummy_converter()
         self.bond_converter = bond_converter or self._get_dummy_converter()
+        self.state_converter = state_converter or self._get_dummy_converter()
         self.graph_data_name = ["atom", "bond", "state", 'atom_nbr_idx']
 
     def convert(self, structure: Structure, state_attributes: List = None) -> Dict:
@@ -77,18 +110,39 @@ class _StructureGraph(BaseFeature):
         if state_attributes is not None:
             state_attributes = np.array(state_attributes)
         else:
-            state_attributes = np.array([[0.0, 0.0]], dtype="float32")
-
-        center_indices, atom_nbr_idx, _, bonds = self.get_bond_fea(structure)
+            state_attributes = np.array([0.0, 0.0], dtype="float32")
+        if isinstance(self.state_converter, DummyConverter):
+            pass
+        else:
+            state_attributes = np.concatenate(
+                (state_attributes, np.array(self.state_converter.convert(structure)).ravel()))
+        center_indices, center_prop, atom_nbr_idx, bond_states, bonds = self.get_bond_fea(structure)
+        if self.return_bonds == "all":
+            bondss = np.concatenate((bond_states, bonds), axis=-1) if bonds is not None else bond_states
+        elif self.return_bonds == "bonds":
+            if bonds is not None:
+                bondss = bonds
+            else:
+                raise TypeError
+        elif self.return_bonds == "bonds_state":
+            bondss = bond_states
+        else:
+            raise NotImplementedError()
         atoms = self.get_atom_fea(structure)
+        atoms = [atoms[int(i)] for i in center_indices]
         atoms = self.atom_converter.convert(atoms)
-        bonds = self.bond_converter.convert(bonds)
+        bonds = self.bond_converter.convert(bondss)
+
+        if center_prop.shape[1] > 1:
+            atoms = np.concatenate((np.array(atoms), center_prop), axis=1)
+
         return {"atom": atoms, "bond": bonds, "state": state_attributes, "atom_nbr_idx": atom_nbr_idx}
 
     @staticmethod
     def get_atom_fea(structure: Structure) -> List[Any]:
         """
-        Get atom features from structure, may be overwritten
+        Get atom features from structure, may be overwritten for you self
+        # todo ding yi zi ji de yuan zi
 
         Args:
             structure: (Pymatgen.Structure) pymatgen structure
@@ -96,11 +150,12 @@ class _StructureGraph(BaseFeature):
         Returns:
             List of atomic numbers
         """
-        return [i.specie.Z for i in structure]
+        return get_atom_fea_number(structure)
 
     def get_bond_fea(self, structure: Structure):
         """
         Get atom features from structure, may be overwritten
+        # todo ding yi zi ji de jian.
 
         Args:
             structure: (Pymatgen.Structure) pymatgen structure
@@ -120,7 +175,7 @@ class _StructureGraph(BaseFeature):
         return self.convert(structure, *args, **kwargs)
 
     @staticmethod
-    def _get_dummy_converter() -> "DummyConverter":
+    def _get_dummy_converter() -> DummyConverter:
         return DummyConverter()
 
     def _transform(self, structures: List[Structure], state_attributes: List = None):
@@ -151,7 +206,7 @@ class _StructureGraph(BaseFeature):
                                      tq=True, batch_size=self.batch_size)
 
             ret, self.support_ = zip(*rets)
-            return ret
+        return ret
 
     def get_flat_data(self, graphs: List[Dict]) -> tuple:
         """
@@ -203,14 +258,25 @@ class _StructureGraphFixedRadius(_StructureGraph):
     def __init__(
             self,
             nn_strategy: Union[str, NearNeighbors] = "MinimumDistanceNNAll",
+            bond_generator: [_BaseEnvGet, str] = None,
             atom_converter: Converter = None,
             bond_converter: Converter = None,
+            state_converter: Converter = None,
             cutoff: float = 7.0,
             **kwargs
     ):
-        super(_StructureGraphFixedRadius, self).__init__(nn_strategy, atom_converter, bond_converter, **kwargs)
+        super(_StructureGraphFixedRadius, self).__init__(nn_strategy=nn_strategy, bond_generator=bond_generator,
+                                                         atom_converter=atom_converter,
+                                                         bond_converter=bond_converter, state_converter=state_converter,
+                                                         **kwargs)
         self.cutoff = cutoff
-        self.bond_generator = BaseBondGet(self.cutoff)
+        if bond_generator is None:
+            self.bond_generator = BaseNNGet(self.nn_strategy, cutoff=cutoff)
+        elif isinstance(bond_generator, str):
+            self.bond_generator = {"BaseNNGet": BaseNNGet, "BaseDesGet": BaseDesGet}[bond_generator](self.nn_strategy,
+                                                                                                     cutoff=cutoff)
+        else:
+            self.bond_generator = bond_generator
 
 
 class CrystalGraph(_StructureGraphFixedRadius):
@@ -233,6 +299,7 @@ class CrystalGraph(_StructureGraphFixedRadius):
             nn_strategy: Union[str, NearNeighbors] = "MinimumDistanceNNAll",
             atom_converter: Converter = None,
             bond_converter: Converter = None,
+            state_converter: Converter = None,
             cutoff: float = 7.0,
             **kwargs
     ):
@@ -246,8 +313,8 @@ class CrystalGraph(_StructureGraphFixedRadius):
             cutoff (float): cutoff radius
         """
         super().__init__(
-            nn_strategy=nn_strategy, atom_converter=atom_converter, bond_converter=bond_converter, cutoff=cutoff,
-            **kwargs)
+            nn_strategy=nn_strategy, atom_converter=atom_converter, bond_converter=bond_converter,
+            state_converter=state_converter, cutoff=cutoff, **kwargs)
 
 
 class CrystalGraphWithBondTypes(_StructureGraph):
@@ -273,6 +340,8 @@ class CrystalGraphWithBondTypes(_StructureGraph):
             nn_strategy: Union[str, NearNeighbors] = "VoronoiNN",
             atom_converter: Converter = None,
             bond_converter: Converter = None,
+            state_converter: Converter = None,
+            return_bonds="bonds",
             **kwargs
     ):
         """
@@ -286,6 +355,7 @@ class CrystalGraphWithBondTypes(_StructureGraph):
             bond_converter = AtomPymatgenPropMap("is_metal", func=None, search_tp="number")
 
         super().__init__(nn_strategy=nn_strategy, atom_converter=atom_converter, bond_converter=bond_converter,
+                         state_converter=state_converter, return_bonds=return_bonds,
                          **kwargs)
 
 
@@ -310,6 +380,7 @@ class CrystalGraphDisordered(_StructureGraphFixedRadius):
             nn_strategy: Union[str, NearNeighbors] = "MinimumDistanceNNAll",
             atom_converter: Converter = AtomJsonMap(),
             bond_converter: Converter = None,
+            state_converter: Converter = None,
             cutoff: float = 7.0,
             **kwargs
     ):
@@ -325,6 +396,7 @@ class CrystalGraphDisordered(_StructureGraphFixedRadius):
         self.cutoff = cutoff
         super().__init__(
             nn_strategy=nn_strategy, atom_converter=atom_converter, bond_converter=bond_converter, cutoff=self.cutoff,
+            state_converter=state_converter,
             **kwargs)
 
     @staticmethod
@@ -339,7 +411,7 @@ class CrystalGraphDisordered(_StructureGraphFixedRadius):
         Returns:
             a list of site fraction description
         """
-        return [i.species.as_dict() for i in structure.sites]
+        return get_atom_fea_name(structure)
 
 
 class SimpleMolGraph(_StructureGraphFixedRadius):
@@ -364,6 +436,7 @@ class SimpleMolGraph(_StructureGraphFixedRadius):
             nn_strategy: Union[str, NearNeighbors] = "AllAtomPairs",
             atom_converter: Converter = None,
             bond_converter: Converter = None,
+            state_converter: Converter = None,
             **kwargs):
         """
         Args:
@@ -374,4 +447,5 @@ class SimpleMolGraph(_StructureGraphFixedRadius):
         if bond_converter is None:
             bond_converter = BondGaussianConverter(np.linspace(0, 4, 20), 0.5)
         super().__init__(nn_strategy=nn_strategy, atom_converter=atom_converter, bond_converter=bond_converter,
+                         state_converter=state_converter,
                          **kwargs)
